#!/usr/bin/env python3

import requests
import getpass
import argparse
import sys
import json
import os
import pkg_resources

from myfpl.live import liveRunner
from myfpl.gameweek import gwRunner
from myfpl.team import teamRunner
from myfpl.fixtures import fixtureRunner

team_id = None

config_path = pkg_resources.resource_filename(
    "myfpl", os.path.join(".config", "config.json"))


def addCli():
    parser = argparse.ArgumentParser(
        description="The myfpl cli allows you to retrieve information about your fpl team.", prog="myfpl",
        epilog="\n"
    )
    parser.add_argument("-g", "--gameweek", action="store_true", dest="gameweek",
                        help="See how your gameweek is going by viewing your real time score, adjusted for bonus and substitutions.")
    parser.add_argument("-t",
                        "--team", action="store_true", dest="team", help="Plan for future gameweeks by viewing transfers made, chips avialable, currently selected team and more. Since your team line-up is private, your Fpl password and email are required. Your email and team id will be cached locally for future use, but never your password. You can clear cache with myfpl -c.")
    parser.add_argument("-l",
                        "--live", action="store_true", dest="live", help="See other people's teams and your league standings before fpl updates. All standings and teams are based on real time scores, which are adjusted for bonus and substitutions. Output also includes every players captaincy choice and transfer hits. It can be used on leagues of any size, inclduing the overall league.")

    parser.add_argument("-c",
                        "--clear", action="store_true", dest="clear", help="Remove your email and team id stored in config.json.")

    parser.add_argument("-f",
                        "--fixtures", action="store_true", dest="fix", help="Get real time scores from PL fixtures.")

    if len(sys.argv) <= 1:
        parser.print_help()
        print()
        sys.exit(0)

    try:
        args = parser.parse_args()
    except:
        parser.print_help()
        print()
        sys.exit(0)

    return args


def handleTeamId():
    global team_id
    flag = False
    with open(config_path, 'r+') as jf:
        config_data = json.load(jf)

        if config_data["team_id"] == "":  # Enter your team id
            flag = True

            while True:
                print("Enter team ID: ", end='')
                try:
                    temp = input()
                    temp = int(temp)
                    if temp <= 0:
                        print("please enter a valid team id.")
                        continue
                    team_id = temp
                    config_data["team_id"] = team_id
                    break
                except KeyboardInterrupt:
                    print()
                    sys.exit(0)
                except:
                    print("please enter a valid team id.")

        else:
            team_id = int(config_data["team_id"])

        if flag:
            jf.seek(0, 0)
            json.dump(config_data, jf, ensure_ascii=False, indent=4)

    return flag


def handleLogin(credentials):
    global team_id
    flag = False
    with open(config_path, 'r+') as jf:
        config_data = json.load(jf)

        if config_data["team_id"] == "":  # Enter your team id
            print("Since your team line-up is private, your Fpl password and email are required. Your email and team id will be cached locally for future use, but never your password. You can clear the cache with myfpl -c.\n")
            flag = True

            while True:
                print("Enter team ID: ", end='')
                try:
                    temp = input()
                    temp = int(temp)
                    if temp <= 0:
                        print("please enter a valid team id.")
                        continue
                    team_id = temp
                    config_data["team_id"] = team_id
                    break
                except KeyboardInterrupt:
                    print()
                    sys.exit(0)
                except:
                    print("please enter a valid team id.")

        else:
            team_id = int(config_data["team_id"])

        if config_data["email"] == "":  # Handle email
            flag = True

            while True:
                print("Enter email: ", end='')
                email = input()

                credentials['login'] = email
                config_data["email"] = email
                break

        else:
            credentials['login'] = config_data["email"]

        if flag:
            jf.seek(0, 0)
            json.dump(config_data, jf, ensure_ascii=False, indent=4)

    credentials['password'] = getpass.getpass()

    return flag


def clearConfig():
    with open(config_path, 'w') as jf:
        config_data = {
            "email": "",
            "team_id": ""
        }
        json.dump(config_data, jf, ensure_ascii=False, indent=4)


def main():
    global team_id

    credentials = {'password': None,
                   'login': None,
                   'redirect_uri': 'https://fantasy.premierleague.com/a/login',
                   'app': 'plfpl-web'
                   }

    args = addCli()
    session = requests.session()

    # Api fetched for all scripts.
    bootstrap_url = 'https://fantasy.premierleague.com/api/bootstrap-static/'
    get_data_bootstrap = session.get(bootstrap_url).json()

    # Shoudl not ask for password.
    if args.gameweek:

        handleTeamId()

        entry_api = 'https://fantasy.premierleague.com/api/entry/%s/' % team_id
        get_data_entry = session.get(entry_api).json()

        # Positive integer provided is too large clear data.
        if len(get_data_entry) == 1:
            print("Bad team id, please try again.")
            clearConfig()
            sys.exit(0)

        if get_data_entry["current_event"] == None:
            print(
                "Fpl api/entry is not available currently, please try again when season returns.")
            sys.exit(0)

        # gw_Team API will always work no login needed.
        gw_team_api = 'https://fantasy.premierleague.com/api/entry/%s/event/%s/picks/' % (
            team_id, get_data_entry["current_event"])
        get_gw_team = session.get(gw_team_api).json()

        live_points_url = "https://fantasy.premierleague.com/api/event/%s/live/#/" % get_data_entry["current_event"]
        get_live_points = session.get(live_points_url).json()

        # testing
        # get_live_points = None
        # with open(config_path) as jf:
        #     get_live_points = json.load(jf)
        # testing

        gwRunner(session, get_gw_team, get_data_bootstrap,
                 get_data_entry, get_live_points)

    # Should not ask for password.
    elif args.live:

        handleTeamId()
        entry_api = 'https://fantasy.premierleague.com/api/entry/%s/' % team_id
        get_data_entry = session.get(entry_api).json()

        if len(get_data_entry) == 1:
            print("Bad team id, please try again.")
            clearConfig()
            sys.exit(0)

        if get_data_entry["current_event"] == None:
            print(
                "Premier League not in season, please come when season returns.")
            sys.exit(0)

        live_points_url = "https://fantasy.premierleague.com/api/event/%s/live/#/" % get_data_entry["current_event"]
        get_live_points = session.get(live_points_url).json()

        fixture_url = "https://fantasy.premierleague.com/api/fixtures/?event=%s#/" % get_data_entry["current_event"]
        get_gw_fixture = session.get(fixture_url).json()

        liveRunner(session, get_data_entry,
                   get_data_bootstrap, get_live_points, get_gw_fixture)

    # Password needed
    elif args.team:
        curr_gw = None
        for k in range(len(get_data_bootstrap["events"])):
            if get_data_bootstrap["events"][k]["is_current"] == True:
                curr_gw = k + 1

        # Handle errors
        if (curr_gw) == None:
            print("Premier League not in season, please come when season returns.")
            sys.exit(0)

        handleLogin(credentials)

        session.post('https://users.premierleague.com/accounts/login/',
                     data=credentials)
        credentials['login'] = None
        credentials['password'] = None
        team_api = 'https://fantasy.premierleague.com/api/my-team/%s/' % (
            team_id)

        get_data_team = session.get(team_api).json()

        if len(get_data_team) == 1:  # If bad passowrd, emial or team_id. Always 200 response,
            print("Wrong login information, please try again.")
            clearConfig()
            sys.exit(0)

        teamRunner(session, team_id, get_data_team,
                   get_data_bootstrap, curr_gw)

    elif args.fix:
        curr_gw = None
        for k in range(len(get_data_bootstrap["events"])):
            if get_data_bootstrap["events"][k]["is_current"] == True:
                curr_gw = k + 1

        # Handle errors
        if (curr_gw) == None:
            print("Premier League not in season, please come back when season returns.")
            sys.exit(0)

        fixture_url = "https://fantasy.premierleague.com/api/fixtures/?event=%s#/" % curr_gw
        get_gw_fixture = session.get(fixture_url).json()

        # testing
        # get_gw_fixture = None
        # with open(config_path) as jf:
        #     get_gw_fixture = json.load(jf)
        # testing

        if not get_gw_fixture:
            print("No fixtures currently available, please try again later.")
        else:
            fixtureRunner(session, get_gw_fixture, get_data_bootstrap, curr_gw)

    if args.clear:
        clearConfig()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

from collections import OrderedDict
from myfpl.gameweek import printGwTeam
import sys


# Globals
league_map = OrderedDict()
live_points_cache = {}  # Used to cache api/live
# Used to cache api/bootstrap. Will store templates that are filled out with team specific info.
player_cache = {}
sp = ' '


def liveRunner(session, get_data_entry, get_data_bootstrap, get_live_points, get_gw_fixture):
    global league_map
    print("\nClassic Leagues:\n")
    print('{0: >54}'.format('Prev'), '{0:>10}'.format('Curr'),
          '{0: >13}'.format('Rank'), '{0: >16}'.format('Leader'))
    print("League Name", '{0:>42}'.format('Rank'), '{0: >10}'.format(
        'Rank'), '{0: >13}'.format('Diff'), '{0: >17}'.format("Points\n"))

    for i in range(len(get_data_entry['leagues']['classic'])):
        league_id = get_data_entry['leagues']['classic'][i]['id']
        league_name = get_data_entry['leagues']['classic'][i]['name']
        previous_rank = get_data_entry['leagues']['classic'][i]['entry_last_rank']
        current_rank = get_data_entry['leagues']['classic'][i]["entry_rank"]
        rank_difference = previous_rank - current_rank

        league_api = 'https://fantasy.premierleague.com/api/leagues-classic/%s/standings/' % league_id
        get_data_league = session.get(league_api).json()

        if len(get_data_league['standings']['results']) == 0:
            print(
                "Fpl end-point api/leagues-classic, is not working, please try again later.")
            return

        league_leader = get_data_league['standings']['results'][0]['total']
        league_difference = league_leader - \
            get_data_entry['summary_overall_points']

        # Create league hashmap. Value is a tuple of name and league id.
        league_map[i] = (league_name, league_id)

        print("(%d) %-45s %-10d %-13d %-14d %d (%d)" % (i, league_name, previous_rank, current_rank,
                                                        rank_difference, league_leader, league_difference))

    while True:
        print(
            "\nEnter the ID (left of name) of the league you want live standings for or q to exit:")
        try:
            ID = input()
            if ID == 'q':
                break
            ID = int(ID)
            if ID not in league_map:
                continue
            else:
                process_league(ID, session, get_data_entry,
                               get_data_bootstrap, get_live_points)

        except KeyboardInterrupt:
            print()
            sys.exit(0)

        except:
            continue

    print()


def process_league(ID, session, get_data_entry, get_data_bootstrap, get_live_points):
    global league_map, live_points_cache, player_cache
    # Orderd dict for sorting efficiency
    user_map = OrderedDict()
    page_num = 1
    limit_page = 4

    league_api = 'https://fantasy.premierleague.com/api/leagues-classic/%s/standings/?page_new_entries=1&page_standings=%s&phase=1' % (
        league_map[ID][1], page_num)
    get_data_league = session.get(league_api).json()
    gameweek = get_data_entry["current_event"]

    while True:
        for i in range(len(get_data_league["standings"]["results"])):
            team_id = get_data_league["standings"]["results"][i]["entry"]
            # User id
            user_id = (get_data_league["standings"]["results"][i]["entry_name"] +
                       ' - ' + get_data_league["standings"]["results"][i]["player_name"])
            # gw_Team API
            gw_team_api = 'https://fantasy.premierleague.com/api/entry/%s/event/%s/picks/' % (
                team_id, gameweek)
            get_gw_team = session.get(gw_team_api).json()
            # get old gw points - costly try and find improvement in future
            gw_old_points = 0
            if gameweek > 1:
                gw_team_old_api = 'https://fantasy.premierleague.com/api/entry/%s/event/%s/picks/' % (
                    team_id, gameweek - 1)
                get_gw_old_team = session.get(gw_team_old_api).json()
                gw_old_points = get_gw_old_team["entry_history"]["total_points"]

            # Captain points(already doubled), Name, triple (boolean)
            captain_id_name = [None, None, False]
            speed_gw_points = 0
            player_list = {"starting": [],
                           "bench": [], "formation": [0, 0, 0, 0]}

            printGwTeam(session, get_gw_team, get_data_bootstrap,
                        get_data_entry, get_live_points, player_list, live_points_cache, player_cache)

            for pl in player_list["starting"]:
                gw_points = live_points_cache[pl["ID"]
                                              ]['stats']['total_points'] #Get actual points.
                gw_points += pl["tot_gw_points"] #Add bonus
                if pl["multiplier"] == 2:
                    gw_points *= 2
                    captain_id_name[0], captain_id_name[1], captain_id_name[2] = gw_points, pl["name"], False
                elif pl["multiplier"] == 3:
                    gw_points *= 3
                    captain_id_name[0], captain_id_name[1], captain_id_name[2] = gw_points, pl["name"], True
                
                pl["tot_gw_points"] = gw_points
                speed_gw_points += gw_points
            
            for pl in player_list["bench"]:
                gw_points = live_points_cache[pl["ID"]]['stats']['total_points']
                gw_points += pl["tot_gw_points"]  # Include bonus.
                pl["tot_gw_points"] = gw_points

            # Have Team name + person name of as key. Value is a tuple:
            # (gw-1 points + gw points - hits, gw points - hits, previous rank, hits, gw team, player_list)
            user_map[user_id] = (speed_gw_points + gw_old_points - get_gw_team["entry_history"]["event_transfers_cost"], speed_gw_points - get_gw_team["entry_history"]["event_transfers_cost"],
                                 get_data_league["standings"]["results"][i]["rank"],
                                 -get_gw_team["entry_history"]["event_transfers_cost"],
                                 captain_id_name, get_gw_team, player_list)

        # Only print first 200 entries. Then ask if want to keep going
        if page_num == limit_page or len(get_data_league['standings']['results']) == 0:
            # Ask if keep going, as there exsits a next page.
            if len(get_data_league['standings']['results']) != 0 and page_num == limit_page:
                print(
                    "\nEnter 'go' to see output or enter anything else to process another 200 players:")
                try:
                    temp = input()
                    if temp == 'go':
                        break
                    else:
                        page_num += 1
                        limit_page += 4
                        league_api = 'https://fantasy.premierleague.com/api/leagues-classic/%s/standings/?page_new_entries=1&page_standings=%s&phase=1' % (
                            league_map[ID][1], page_num)
                        get_data_league = session.get(league_api).json()

                except KeyboardInterrupt:
                    print()
                    sys.exit(0)

                except:
                    page_num += 1
                    limit_page += 4
                    league_api = 'https://fantasy.premierleague.com/api/leagues-classic/%s/standings/?page_new_entries=1&page_standings=%s&phase=1' % (
                        league_map[ID][1], page_num)
                    get_data_league = session.get(league_api).json()

            else:
                break

        else:
            page_num += 1
            league_api = 'https://fantasy.premierleague.com/api/leagues-classic/%s/standings/?page_new_entries=1&page_standings=%s&phase=1' % (
                league_map[ID][1], page_num)
            get_data_league = session.get(league_api).json()

    # Perform sorting: Use selection sort as stable and exploit psudo orderd property of league table. Faster than python nlogn library quicksort.
    user_list = list(user_map.items())
    for k in range(1, len(user_list)):
        key = user_list[k]
        w = k - 1
        while w >= 0 and key[1][0] > user_list[w][1][0]:
            user_list[w + 1] = user_list[w]
            w -= 1
        user_list[w + 1] = key

    print()
    print('{0: >49}'.format('Curr'), '{0:>9}'.format('Prev'),
          '{0: >9}'.format('Rank'), '{0: >43}'.format('Net'))
    print('{0: <41}'.format('Name'), '{0:>7}'.format('Rank'), '{0: >9}'.format('Rank'), '{0: >9}'.format(
        'Diff'), '{0: >11}'.format('Captain'), '{0: >24}'.format('Hits'), '{0: >10}'.format("Points\n"))

    for i in range(len(user_list)):
        print("%-44s %-9d %-9d %-8d %-27s %-7d %-3d (%d)" % (user_list[i][0], i + 1, user_list[i][1][2], user_list[i][1][2] - (i + 1), ((user_list[i][1][4][1] + ' ('+str(
            user_list[i][1][4][0]) + ')' + '(TC)') if user_list[i][1][4][2] else user_list[i][1][4][1] + ' ('+str(user_list[i][1][4][0]) + ')'), user_list[i][1][3], user_list[i][1][1], user_list[i][1][0]))

    while True:
        print("\nView a players team by entering their current rank:")
        try:
            ID = int(input())
            if 1 <= ID <= len(user_list):
                player_list = user_list[ID - 1][1][6]
                get_gw_team = user_list[ID - 1][1][5]
                speed_gw_points = 0
                print("\n" + user_list[ID - 1][0])
                print("Gameweek:", get_data_entry["current_event"])
                print()
                print('{0: >42}'.format("Points"), '{0:>13}'.format(
                    '+/-'), '{0: >9}'.format('Chance'))
                print('Name', '{0:>35}'.format('(GW)'), '{0: >9}'.format('Price'), '{0: >6}'.format(
                    '(GW)'), '{0: >8}'.format('NextGW'), '{0: >7}'.format("News\n"))

                # printGwTeam(session, get_gw_team,
                #             get_data_bootstrap, get_data_entry, get_live_points, player_list, live_points_cache, player_cache)

                for pl in player_list["starting"]:
                    gw_points = pl["tot_gw_points"]
                        
                    speed_gw_points += gw_points
                    print("%-25s %-9s %-8d %-7.1f %-6.1f %-8d %s" %
                          (pl["name"], pl["player_status"], pl["tot_gw_points"], pl["price"], pl["price_change"], pl["next_round"], pl["news"]))

                for pl in player_list["bench"]:
                    print("%-25s %-9s %-8d %-7.1f %-6.1f %-8d %s" %
                          (pl["name"], pl["player_status"], pl["tot_gw_points"], pl["price"], pl["price_change"], pl["next_round"], pl["news"]))

                print("\n\nGameweek points: %d (%d)       \tOverall points: %-7d" %
                      (speed_gw_points, get_gw_team["entry_history"]["event_transfers_cost"],
                       get_gw_team["entry_history"]["total_points"]))

                print("Gameweek rank:   %-7s\tOverall rank:   %-7s\n" %
                      (get_gw_team["entry_history"]["rank"], get_gw_team["entry_history"]["overall_rank"]))

        except KeyboardInterrupt:
            print()
            sys.exit(0)

        except:
            break



#!/usr/bin/env python3

def gwRunner(session, get_gw_team, get_data_bootstrap, get_data_entry, get_live_points):

    sp = ' '
    player_list = {"starting": [], "bench": [], "formation": [0, 0, 0, 0]}
    live_points_cache = {}
    player_cache = {}
    speed_gw_points = 0
    print()

    print("Gameweek:", get_data_entry["current_event"])
    print()
    print('{0: >42}'.format("Points"), '{0:>13}'.format(
        '+/-'), '{0: >9}'.format('Chance'))
    print('Name', '{0:>35}'.format('(GW)'), '{0: >9}'.format('Price'), '{0: >6}'.format(
        '(GW)'), '{0: >8}'.format('NextGW'), '{0: >7}'.format("News\n"))

    printGwTeam(session, get_gw_team, get_data_bootstrap,
                get_data_entry, get_live_points, player_list, live_points_cache, player_cache)

    for pl in player_list["starting"]:
        gw_points = live_points_cache[pl["ID"]]['stats']['total_points']
        gw_points += pl["tot_gw_points"]  # Include bonus.
        if pl["multiplier"] == 2:
            gw_points *= 2
        elif pl["multiplier"] == 3:
            gw_points *= 3
        
        pl["tot_gw_points"] = gw_points
        speed_gw_points += gw_points
        print("%-25s %-9s %-8d %-7.1f %-6.1f %-8d %s" %
              (pl["name"], pl["player_status"], pl["tot_gw_points"], pl["price"], pl["price_change"], pl["next_round"], pl["news"]))

    for pl in player_list["bench"]:
        gw_points = live_points_cache[pl["ID"]]['stats']['total_points']
        gw_points += pl["tot_gw_points"]  # Include bonus.
        pl["tot_gw_points"] = gw_points
        print("%-25s %-9s %-8d %-7.1f %-6.1f %-8d %s" %
              (pl["name"], pl["player_status"], pl["tot_gw_points"], pl["price"], pl["price_change"], pl["next_round"], pl["news"]))

    print("\n\nGameweek points: %d (%d)       \tOverall points: %-7d" %
          (speed_gw_points, get_gw_team["entry_history"]["event_transfers_cost"],
           get_gw_team["entry_history"]["total_points"] + speed_gw_points))

    print("Gameweek rank:   %-7s\tOverall rank:   %-7s\n" %
          (get_gw_team["entry_history"]["rank"], get_gw_team["entry_history"]["overall_rank"]))

    print("\nOfficial league standings as per fpl; use myfpl -l for real time standings.\n")

    print("\nH2H Leagues:\n")
    print(sp*40 + "Prev" + sp*7 + "Curr" + sp*7 + "Rank" + sp*7 + "Leader")
    print("League Name" + sp*29 + "Rank" + sp*7 +
          "Rank" + sp*7 + "Diff" + sp*7 + "Points\n")

    for i in range(len(get_data_entry['leagues']['h2h'])):
        league_id = get_data_entry['leagues']['h2h'][i]['id']
        league_name = get_data_entry['leagues']['h2h'][i]['name']
        previous_rank = get_data_entry['leagues']['h2h'][i]['entry_last_rank']
        current_rank = get_data_entry['leagues']['h2h'][i]['entry_rank']
        rank_difference = get_data_entry['leagues']['h2h'][i]['entry_last_rank'] - current_rank

        league_api = 'https://fantasy.premierleague.com/api/leagues-h2h/%s/standings/' % league_id
        get_data_league = session.get(league_api).json()
        if len(get_data_league['standings']['results']) == 0:
            print(
                "Fpl end-point api/leagues-h2h, is not working, please try again later.")
            return

        league_leader = get_data_league['standings']['results'][0]['total']
        point_differential = league_leader - \
            get_data_league['standings']['results'][current_rank - 1]['total']

        print("%-39s %-10d %-10d %-10d %d (%d)" %
              (league_name, previous_rank, current_rank, rank_difference, league_leader, point_differential))

    print("\n\nClassic Leagues:\n")
    print(sp*40 + "Prev" + sp*7 + "Curr" + sp*7 + "Rank" + sp*7 + "Leader")
    print("League Name" + sp*29 + "Rank" + sp*7 +
          "Rank" + sp*7 + "Diff" + sp*7 + "Points\n")

    for i in range(len(get_data_entry['leagues']['classic'])):
        league_id = get_data_entry['leagues']['classic'][i]['id']
        league_name = get_data_entry['leagues']['classic'][i]['name']
        previous_rank = get_data_entry['leagues']['classic'][i]['entry_last_rank']
        current_rank = get_data_entry['leagues']['classic'][i]["entry_rank"]
        rank_difference = previous_rank - current_rank

        league_api = 'https://fantasy.premierleague.com/api/leagues-classic/%s/standings/' % league_id
        get_data_league = session.get(league_api).json()

        if len(get_data_league['standings']['results']) == 0:
            print(
                "Fpl end-point api/leagues-classic, is not working, please try again later.")
            return

        league_leader = get_data_league['standings']['results'][0]['total']
        league_difference = league_leader - \
            get_data_entry['summary_overall_points']

        print("%-39s %-10d %-10d %-10d %d (%d)" % (league_name, previous_rank, current_rank,
                                                   rank_difference, league_leader, league_difference))

    print("\n")


def printGwTeam(session, get_gw_team, get_data_bootstrap, get_data_entry, get_live_points, player_list, live_points_cache, player_cache):

    # Key team, value players.
    team_list = {}

    # Use live point to get gw points as well.

    for j in range(len(get_gw_team['picks'])):
        ID = get_gw_team['picks'][j]['element']
        multiplier = get_gw_team['picks'][j]['multiplier']
        vice_captain = get_gw_team['picks'][j]['is_vice_captain']
        team_position = get_gw_team['picks'][j]['position']

        if multiplier == 2:
            player_status = "(C)"
        elif multiplier == 3:
            player_status = "(TC)"
        elif vice_captain == True:
            player_status = "(VC)"
        elif multiplier == 0:
            player_status = "(Bench)"
        else:
            player_status = ""

        if ID in player_cache:
            temp = player_cache[ID].copy()
            temp["player_status"] = player_status
            temp["team_position"] = team_position
            temp["multiplier"] = multiplier

            team = temp["team"]
            if not team in team_list:
                team_list[team] = ID

            if player_status == "(Bench)":
                player_list["bench"].append(temp)
            else:
                player_list["starting"].append(temp)
                player_list["formation"][temp["element_type"] - 1] += 1
        else:
            for i in range(len(get_data_bootstrap['elements'])):
                if get_data_bootstrap['elements'][i]['id'] == ID:
                    name = get_data_bootstrap['elements'][i]['web_name']

                    price = get_data_bootstrap['elements'][i]['now_cost'] / 10
                    price_change = get_data_bootstrap['elements'][i]['cost_change_event'] / 10
                    next_round = get_data_bootstrap['elements'][i]['chance_of_playing_next_round']
                    if next_round == None:
                        next_round = 100
                    news = get_data_bootstrap['elements'][i]['news']

                    team = get_data_bootstrap['elements'][i]["team"]
                    if not team in team_list:
                        team_list[team] = ID

                    if player_status == "(Bench)":
                        player_list["bench"].append({
                            "name": name,
                            "player_status": player_status,
                            "tot_gw_points": 0,  # Added later.
                            "price": price,
                            "price_change": price_change,
                            "next_round": next_round,
                            "news": news,
                            "team": team,
                            "ID": ID,
                            "element_type": get_data_bootstrap['elements'][i]["element_type"],
                            "team_position": team_position,
                            "multiplier": multiplier
                        })
                        player_cache[ID] = {
                            "name": name,
                            "player_status": None,
                            "tot_gw_points": 0,  # Added later.
                            "price": price,
                            "price_change": price_change,
                            "next_round": next_round,
                            "news": news,
                            "team": team,
                            "ID": ID,
                            "element_type": get_data_bootstrap['elements'][i]["element_type"],
                            "team_position": None,  # Team specific.
                            "multiplier": None  # Team specific
                        }
                    else:
                        player_list["starting"].append({
                            "name": name,
                            "player_status": player_status,
                            "tot_gw_points": 0,  # Added later.
                            "price": price,
                            "price_change": price_change,
                            "next_round": next_round,
                            "news": news,
                            "team": team,
                            "ID": ID,
                            "element_type": get_data_bootstrap['elements'][i]["element_type"],
                            "team_position": team_position,
                            "multiplier": multiplier
                        })
                        player_cache[ID] = {
                            "name": name,
                            "player_status": None,
                            # Added later.
                            "tot_gw_points": 0,
                            "price": price,
                            "price_change": price_change,
                            "next_round": next_round,
                            "news": news,
                            "team": team,
                            "ID": ID,
                            "element_type": get_data_bootstrap['elements'][i]["element_type"],
                            "team_position": None,  # Team specific.
                            "multiplier": None  # Team specific
                        }
                        player_list["formation"][get_data_bootstrap['elements']
                                                 [i]["element_type"] - 1] += 1

                    break

    # Populate live_points_cache here.
    for k in range(len(player_list["bench"])):
        ID = player_list["bench"][k]["ID"]
        if ID not in live_points_cache:
            for i in range(len(get_live_points['elements'])):
                if get_live_points['elements'][i]['id'] == ID:
                    live_points_cache[ID] = get_live_points['elements'][i]
                    break

    for k in range(len(player_list["starting"])):
        ID = player_list["starting"][k]["ID"]
        if ID not in live_points_cache:
            for i in range(len(get_live_points['elements'])):
                if get_live_points['elements'][i]['id'] == ID:
                    live_points_cache[ID] = get_live_points['elements'][i]
                    break

    if not get_data_bootstrap["events"][get_data_entry["current_event"]]["finished"]:
        getFixtuerData(session, get_data_bootstrap, get_data_entry,
                       player_list, team_list, get_live_points, live_points_cache)

    # testing
    # getFixtuerData(session, get_data_bootstrap, get_data_entry,
    #                player_list, team_list, get_live_points, live_points_cache)
    # testing


def getFixtuerData(session, get_data_bootstrap, get_data_entry, player_list, team_list, get_live_points, live_points_cache) -> int:
    fixture_url = "https://fantasy.premierleague.com/api/fixtures/?event=%s#/" % get_data_entry["current_event"]
    get_gw_fixture = session.get(fixture_url).json()

    # testing
    # get_gw_fixture = None
    # with open('./teststuff/test2.json') as jf:
    #     get_gw_fixture = json.load(jf)
    # testing

    finished = {}
    started = {}
    upcoming = {}

    # Iterate over all fixtuers
    for g in range(len(get_gw_fixture)):
        if get_gw_fixture[g]["finished"] == True:
            finished[get_gw_fixture[g]["team_a"]] = get_gw_fixture[g]
            finished[get_gw_fixture[g]["team_h"]] = get_gw_fixture[g]

        elif get_gw_fixture[g]["started"] == True:
            started[get_gw_fixture[g]["team_a"]] = get_gw_fixture[g]
            started[get_gw_fixture[g]["team_h"]] = get_gw_fixture[g]

        else:
            upcoming[get_gw_fixture[g]["team_a"]] = get_gw_fixture[g]
            upcoming[get_gw_fixture[g]["team_h"]] = get_gw_fixture[g]

        # Possibly add bonus points
        if get_gw_fixture[g]["team_a"] in team_list or get_gw_fixture[g]["team_h"] in team_list:
            # Need add provisional bonus points.
            if get_gw_fixture[g]["started"] == True and get_gw_fixture[g]["finished"] == False:

                bp_players = find_prelim_bonus(get_gw_fixture[g])

                # Now we need to check if any of your players are on bonus, if so increase their points.
                for pl in range(len(player_list["starting"])):
                    if player_list["starting"][pl]["ID"] in bp_players:
                        player_list["starting"][pl]["tot_gw_points"] += bp_players[player_list["starting"][pl]["ID"]]

                for pl in range(len(player_list["bench"])):
                    if player_list["bench"][pl]["ID"] in bp_players:
                        player_list["bench"][pl]["tot_gw_points"] += bp_players[player_list["bench"][pl]["ID"]]

    n_subs_made = 0

    for k in range(len(player_list["starting"])):
        player_info = live_points_cache[player_list["starting"][k]["ID"]]
        if player_list["starting"][k]["team"] not in upcoming and not player_info["stats"]["minutes"] > 0:
            replacement_found = make_sub(
                k, player_list, finished, started, upcoming, live_points_cache)
            if replacement_found:
                n_subs_made += 1
        if n_subs_made == 3:
            break


'''
Edge cases:
* Need to maintain the minimum number of players in each position:
    - golakeeper can only replace goal keeper.
    - If playing 3 at the back, and defender not playing. Only another defender can replace that defender.
    - If playing 1 striker, only striker can replace that striker.
    - If playing 3 midfield, only midfielder can replace that midfielder.
'''


def make_sub(index_of_sub, player_list, finished, started, upcoming, live_points_cache) -> bool:
    player_to_swap = player_list["starting"][index_of_sub]
    n_defenders = player_list["formation"][1]
    n_mids = player_list["formation"][2]
    n_attackers = player_list["formation"][3]

    valid_sub_types = set()
    replacement_found = False  # There may be no replacement.

    # We need to use defender sub.
    if n_defenders == 3 and player_to_swap["element_type"] == 2:
        valid_sub_types.add(2)
    elif n_mids == 3 and player_to_swap["element_type"] == 3:
        valid_sub_types.add(3)  # We need to use mid sub.
    elif n_attackers == 1 and player_to_swap["element_type"] == 4:
        valid_sub_types.add(4)  # We need to use attacker sub.
    elif player_to_swap["element_type"] == 1:
        valid_sub_types.add(1)  # We need to use goalie sub.
    else:  # Any type will do.
        valid_sub_types.update([2, 3, 4])

    for j in range(len(player_list["bench"])):
        # make swap.
        if player_list["bench"][j]["team"] not in upcoming and player_list["bench"][j]["element_type"] in valid_sub_types and live_points_cache[player_list["bench"][j]["ID"]]["stats"]["minutes"] > 0:
            replacement_found = True
            # Change formation
            player_list["formation"][player_list["bench"]
                                     [j]["element_type"] - 1] += 1
            player_list["formation"][player_to_swap["element_type"] - 1] -= 1

            # If captain is benched need to make multiplier 2 fro vice captain.
            if player_to_swap["player_status"] == "(C)":
                for k in range(len(player_list["starting"])):
                    if player_list["starting"][k]["player_status"] == "(VC)":
                        player_list["starting"][k]["player_status"] = "(C)"
                        player_list["starting"][k]["multiplier"] = 2
                        break

            elif player_to_swap["player_status"] == "(TC)":
                for k in range(len(player_list["starting"])):
                    if player_list["starting"][k]["player_status"] == "(VC)":
                        player_list["starting"][k]["player_status"] = "(TC)"
                        player_list["starting"][k]["multiplier"] = 3
                        break

            # Update player statuss
            player_list["bench"][j]["player_status"] = ""
            player_to_swap["player_status"] = "(Bench)"

            # Multiplier
            player_list["bench"][j]["multiplier"] = 1
            player_to_swap["multiplier"] = 0

            player_list["bench"][j]["team_position"], player_to_swap["team_position"] = player_to_swap[
                "team_position"], player_list["bench"][j]["team_position"]

            # Need to swap elements in starting and bench array.
            player_list["bench"][j], player_list["starting"][index_of_sub] = player_list["starting"][index_of_sub], player_list["bench"][j]

            break

    # There may be no replacement in a very bad case.
    if not replacement_found:
        return False
    return True


def find_prelim_bonus(fixture):

    h_i, a_i = 0, 0
    bp_players = {}
    n_players = 3
    record = {"last_bps_score": -1, "last_bonus": -1}

    while n_players > 0 and h_i < len(fixture["stats"][9]["h"]) and a_i < len(fixture["stats"][9]["a"]):
        if fixture["stats"][9]["h"][h_i]["value"] > fixture["stats"][9]["a"][a_i]["value"]:
            if fixture["stats"][9]["h"][h_i]["value"] == record["last_bps_score"]:
                bp_players[fixture
                           ["stats"][9]["h"][h_i]["element"]] = record["last_bonus"]
            else:
                bp_players[fixture["stats"][9]["h"]
                           [h_i]["element"]] = n_players
                record["last_bonus"] = n_players
                record["last_bps_score"] = fixture["stats"][9]["h"][h_i]["value"]

            h_i += 1
            n_players -= 1

        elif fixture["stats"][9]["h"][h_i]["value"] < fixture["stats"][9]["a"][a_i]["value"]:
            if fixture["stats"][9]["a"][a_i]["value"] == record["last_bps_score"]:
                bp_players[fixture
                           ["stats"][9]["a"][a_i]["element"]] = record["last_bonus"]
            else:
                bp_players[fixture
                           ["stats"][9]["a"][a_i]["element"]] = n_players
                record["last_bonus"] = n_players
                record["last_bps_score"] = fixture["stats"][9]["a"][a_i]["value"]

            a_i += 1
            n_players -= 1

        elif fixture["stats"][9]["h"][h_i]["value"] == fixture["stats"][9]["a"][a_i]["value"]:

            bp_players[fixture
                       ["stats"][9]["a"][a_i]["element"]] = n_players
            bp_players[fixture
                       ["stats"][9]["h"][h_i]["element"]] = n_players

            n_players -= 2
            h_i += 1
            a_i += 1

    return bp_players



#!/usr/bin/env python3

def teamRunner(session, team_id, get_data_team, get_data_bootstrap, curr_gw):
    sp = ' '

    print('\n')
    print('{0: >47}'.format("+/-"), '{0:>11}'.format('Purchase'),
          '{0: >8}'.format('Chance'))
    print('Name', '{0:>36}'.format('Price'), '{0: >6}'.format('(GW)'), '{0: >7}'.format(
        'Price'), '{0: >11}'.format('NextGW'), '{0: >7}'.format("News\n"))

    for j in range(len(get_data_team['picks'])):
        id = get_data_team['picks'][j]['element']
        multiplier = get_data_team['picks'][j]['multiplier']
        vice_captain = get_data_team['picks'][j]['is_vice_captain']
        position = get_data_team['picks'][j]['position']

        if multiplier == 2:
            player_status = "(C)"
        elif multiplier == 3:
            player_status = "(TC)"
        elif vice_captain == True:
            player_status = "(VC)"
        elif multiplier == 0:
            player_status = "(Bench)"
        else:
            player_status = ""

        for i in range(len(get_data_bootstrap['elements'])):
            if get_data_bootstrap['elements'][i]['id'] == id:
                name = get_data_bootstrap['elements'][i]['web_name']
                price = get_data_bootstrap['elements'][i]['now_cost'] / 10
                price_change = get_data_bootstrap['elements'][i]['cost_change_event'] / 10
                next_round = get_data_bootstrap['elements'][i]['chance_of_playing_next_round']
                if next_round == None:
                    next_round = 100
                news = get_data_bootstrap['elements'][i]['news']

                print("%-25s %-9s %-7.1f %-6.1f %-10.1f %-8d %s" %
                      (name, player_status, price, price_change, (get_data_team['picks'][j]['purchase_price'] / 10), next_round, news))

                break

    print("\nFree transfers: %s" % (get_data_team['transfers']["limit"]))
    print("Transfers used: %s" % get_data_team['transfers']["made"])
    print("Team value: £%.1fM" % (get_data_team['transfers']["value"] / 10))
    print("Money in da bank: £%.1fM" %
          (get_data_team['transfers']["bank"] / 10))

    if len(get_data_team["chips"][0]["played_by_entry"]) == 0:
        if int(get_data_team["chips"][0]["number"]) == 1:
            print("\nWildcard: Yes, use by gameweek %s (%d gameweeks left)" % (
                  get_data_team["chips"][0]["stop_event"], int(get_data_team["chips"][0]["stop_event"]) - curr_gw))
        else:
            print("\nWildcard: Yes, %d gameweeks left" % int(
                get_data_team["chips"][0]["stop_event"]) - curr_gw)
    else:
        print("\nWildcard: No")

    print("Free hit: %s" % ("Yes" if len(
        get_data_team["chips"][1]["played_by_entry"]) == 0 else "No"))
    print("Bench boost: %s" % ("Yes" if len(
        get_data_team["chips"][2]["played_by_entry"]) == 0 else "No"))
    print("Triple captain: %s" % ("Yes" if len(
        get_data_team["chips"][3]["played_by_entry"]) == 0 else "No"))

    # Transfers
    # Latest transfers API
    latest_transfer_url = 'https://fantasy.premierleague.com/api/entry/%s/transfers-latest/' % (
        team_id)
    get_latest_transfer = session.get(latest_transfer_url).json()
    print('\nTransfers:')
    print('Player out' + 21*sp + 'Player in\n')

    for j in range(len(get_latest_transfer)):
        trade = get_latest_transfer[j]
        flag_in = False
        flag_out = False
        name_in = None
        name_out = None

        i = 0
        while flag_out == False or flag_in == False:
            if flag_in == False and get_data_bootstrap['elements'][i]['id'] == trade["element_in"]:
                name_in = get_data_bootstrap['elements'][i]['web_name']
                flag_in = True

            if flag_out == False and get_data_bootstrap['elements'][i]['id'] == trade['element_out']:
                name_out = get_data_bootstrap['elements'][i]['web_name']
                flag_out = True

            i += 1
        print("%-30s %s" % (name_out, name_in))

    print("\n")



#!/usr/bin/env python3

from setuptools import setup
from os import path

with open("README.md", 'r') as f:
    long_description = f.read()

setup(
    name='myfpl',
    version='2.4.1',
    description='A useful module',
    license="MIT",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='trozler',
    author_email='tony.rosler246@gmail.com',
    url="https://github.com/trozler/myfpl",
    packages=['myfpl'],
    keywords=['fpl', 'fantasy premier league',
              'cli', 'command line'],
    install_requires=['requests'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["myfpl=myfpl.__main__:main"]},
    include_package_data=True,
    package_data={
        ".config": [path.join(".config", "config.json")]
    }
)



#!/usr/bin/env python3

import sys
from myfpl.gameweek import find_prelim_bonus
from datetime import datetime, timezone

global_cnt = 0
print_delta = 0


def fixtureRunner(session, get_gw_fixture, get_data_bootstrap, curr_gw):
    print("\nGameweek:", curr_gw)
    print()

    team_list = {}  # Key is team id and val is number.

    for k in range(len(get_data_bootstrap["teams"])):
        team_list[get_data_bootstrap["teams"][k]
                  ["id"]] = get_data_bootstrap["teams"][k]["name"]

    finished = []
    started = []
    upcoming = []
    player_cache = {}

    for g in range(len(get_gw_fixture)):
        if get_gw_fixture[g]["finished_provisional"] == True:
            finished.append(get_gw_fixture[g])

        elif get_gw_fixture[g]["started"] == True:
            started.append(get_gw_fixture[g])

        else:  # Upcoming
            upcoming.append(get_gw_fixture[g])

    if finished:
        print_finished(finished, team_list, get_data_bootstrap, player_cache)
        print()

    if started:
        print_started(session, started, team_list, get_data_bootstrap,
                      player_cache, curr_gw)
        print()

    if upcoming:
        print_upcoming(upcoming, team_list, get_data_bootstrap)
        print()


def findNames(home_list, away_list, get_data_bootstrap, player_cache):
    point_scorers = {"home": [], "away": []}

    for k in range(len(home_list)):
        ID = home_list[k]["element"]
        if ID not in player_cache:
            for j in range(len(get_data_bootstrap['elements'])):
                if get_data_bootstrap['elements'][j]['id'] == ID:
                    player_cache[ID] = {
                        "name": get_data_bootstrap['elements'][j]["web_name"],
                        "team_id": get_data_bootstrap['elements'][j]["team"]
                    }
                    point_scorers["home"].append({
                        "id": ID,
                        "n_points": home_list[k]["value"],
                        "name": player_cache[ID]["name"],
                        "team_id": player_cache[ID]["team_id"]
                    })
                    break
        else:
            point_scorers["home"].append({
                "id": ID,
                "n_points": home_list[k]["value"],
                "name": player_cache[ID]["name"],
                "team_id": player_cache[ID]["team_id"]
            })

    for k in range(len(away_list)):
        ID = away_list[k]["element"]
        if ID not in player_cache:
            for j in range(len(get_data_bootstrap['elements'])):
                if get_data_bootstrap['elements'][j]['id'] == ID:
                    player_cache[ID] = {
                        "name": get_data_bootstrap['elements'][j]["web_name"],
                        "team_id": get_data_bootstrap['elements'][j]["team"]
                    }
                    point_scorers["away"].append({
                        "id": ID,
                        "n_points": away_list[k]["value"],
                        "name": player_cache[ID]["name"],
                        "team_id": player_cache[ID]["team_id"]
                    })
                    break
        else:
            point_scorers["away"].append({
                "id": ID,
                "n_points": away_list[k]["value"],
                "name": player_cache[ID]["name"],
                "team_id": player_cache[ID]["team_id"]
            })
    return point_scorers


def print_upcoming(upcoming, team_list, get_data_bootstrap):
    global global_cnt
    print("\nUpcoming:")
    for g in range(len(upcoming)):
        print('\n\n{:<5} {:>17}'.format(
            str(g + global_cnt + 1) + ".", team_list[upcoming[g]["team_h"]]), end="")
        print('{:<12} {} '.format("", "vs"), end="")

        # https://stackoverflow.com/questions/45448083/convert-time-from-utc-to-gmt-with-python
        # https://stackoverflow.com/questions/7065164/how-to-make-an-unaware-datetime-timezone-aware-in-python
        # datetime.strptime("2020-09-12T11:30:00Z", "%Y-%m-%dT%H:%M:%SZ").strftime("%a %d %b %Y, %H:%M GMT.")
        display_time = None
        if sys.version_info.minor < 6:
            display_time = datetime.strptime(
                upcoming[g]["kickoff_time"], "%Y-%m-%dT%H:%M:%SZ").strftime("%a %d %b %Y, %H:%M GMT.")
        else:
            display_time = datetime.strptime(upcoming[g]["kickoff_time"], "%Y-%m-%dT%H:%M:%SZ").replace(
                tzinfo=timezone.utc).astimezone().strftime("%a %d %b %Y, %H:%M %Z.")

        print('{:<14} {:18} {}'.format(
            "", team_list[upcoming[g]["team_a"]], display_time))

    global_cnt += 1


def find_time(session, fixture, get_data_bootstrap, player_cache, curr_gw):
    kickoff_time = datetime.strptime(
        fixture["kickoff_time"], "%Y-%m-%dT%H:%M:%SZ")
    now = datetime.utcnow()
    elapsedTime = now - kickoff_time
    game_time = divmod(elapsedTime.total_seconds(), 60)

    if game_time[0] < 15:  # May in extreme case have no bps points.
        return int(game_time[0])
    else:  # Need to get max time spent on pitch of 3 players.
        live_points_url = "https://fantasy.premierleague.com/api/event/%s/live/#/" % curr_gw
        get_live_points = session.get(live_points_url).json()

        markers = [fixture["stats"][9]["a"][0], fixture["stats"]
                   [9]["h"][0], fixture["stats"][9]["h"][1]]

        time = 0

        for pl in markers:
            ID = pl["element"]
            for i in range(len(get_live_points['elements'])):
                if get_live_points['elements'][i]['id'] == ID:
                    if get_live_points['elements'][i]["stats"]["minutes"] > time:
                        time = get_live_points['elements'][i]["stats"]["minutes"]
                    break
        return time


def print_started(session, started, team_list, get_data_bootstrap, player_cache, curr_gw):
    global global_cnt
    print("\nStarted:")
    for g in range(len(started)):

        # Get the playing time of keeper and 3 defenders, then take mode as gametime.
        display_time = find_time(
            session, started[g], get_data_bootstrap, player_cache, curr_gw)
        # display_time = started[g]["minutes"]

        print('\n\n{:<5} {:>17}'.format(
            str(g + global_cnt + 1) + ".", team_list[started[g]["team_h"]]), end="")
        print('{:<12} {} - {}'.format("",
                                      started[g]["team_h_score"], started[g]["team_a_score"]), end="")
        print('{:<12} {:18} ({} mins)'.format(
            "", team_list[started[g]["team_a"]], display_time))

        for k in [0, 2, 1, 4, 5, 6, 9]:

            h_i, a_i = 0, 0
            h_points = len(started[g]["stats"][k]["h"])
            a_points = len(started[g]["stats"][k]["a"])

            point_scorers = None

            if h_points > 0 or a_points > 0:  # Only print if some points were scored.
                if k == 0:
                    print('{:<36}{}'.format("", "Goals"))
                elif k == 2:
                    print('{:<36}{}'.format("", "Own goals"))
                elif k == 1:
                    print('{:<36}{}'.format("", "Assists"))
                elif k == 4:
                    print('{:<36}{}'.format("", "Penalties missed"))
                elif k == 5:
                    print('{:<36}{}'.format("", "Yellow cards"))
                elif k == 6:
                    print('{:<36}{}'.format("", "Red cards"))
                elif k == 9:
                    print('{:<36}{}'.format("", "Prelim bonus"))

                if k == 9:

                    bonus_points = find_prelim_bonus(started[g])
                    # find_prelim_bonus returns data in wrong format, need to find team and name of player. Before printing.
                    temp_all = [{"value": v, "element": k}
                                for k, v in bonus_points.items()]
                    home_team_id = started[g]["team_h"]
                    away_team_id = started[g]["team_a"]

                    point_scorers = {"home": [], "away": []}

                    for k in range(len(temp_all)):
                        ID = temp_all[k]["element"]
                        if ID in player_cache:
                            team_id = player_cache[ID]["team_id"]
                            if team_id == home_team_id:
                                point_scorers["home"].append({
                                    "id": ID,
                                    "n_points": temp_all[k]["value"],
                                    "name": player_cache[ID]["name"],
                                    "team_id": player_cache[ID]["team_id"]
                                })
                            elif team_id == away_team_id:
                                point_scorers["away"].append({
                                    "id": ID,
                                    "n_points": temp_all[k]["value"],
                                    "name": player_cache[ID]["name"],
                                    "team_id": player_cache[ID]["team_id"]
                                })
                            else:
                                print(
                                    "Error, player ID doesnt belong to any of the teams.")
                                sys.exit(1)
                        else:
                            for j in range(len(get_data_bootstrap['elements'])):
                                if get_data_bootstrap['elements'][j]['id'] == ID:
                                    player_cache[ID] = {
                                        "name": get_data_bootstrap['elements'][j]["web_name"],
                                        "team_id": get_data_bootstrap['elements'][j]["team"]
                                    }

                                    team_id = player_cache[ID]["team_id"]

                                    if team_id == home_team_id:
                                        point_scorers["home"].append({
                                            "id": ID,
                                            "n_points": temp_all[k]["value"],
                                            "name": player_cache[ID]["name"],
                                            "team_id": player_cache[ID]["team_id"]
                                        })

                                    elif team_id == away_team_id:
                                        point_scorers["away"].append({
                                            "id": ID,
                                            "n_points": temp_all[k]["value"],
                                            "name": player_cache[ID]["name"],
                                            "team_id": player_cache[ID]["team_id"]
                                        })

                                    break

                    h_points = len(point_scorers["home"])
                    a_points = len(point_scorers["away"])
                else:
                    point_scorers = findNames(started[g]["stats"][k]["h"],
                                              started[g]["stats"][k]["a"], get_data_bootstrap, player_cache)

                while h_i < h_points or a_i < a_points:
                    away_yes = False
                    home_yes = False

                    if h_i < h_points:
                        home_yes = True
                        print('{:>21} {}'.format(
                            point_scorers["home"][h_i]["name"], point_scorers["home"][h_i]["n_points"]), end='')
                        h_i += 1

                    if a_i < a_points:
                        if home_yes:  # Just printed a home player need alignment for away.
                            print('{:<31}{} {}'.format("",
                                                       point_scorers["away"][a_i]["name"], point_scorers["away"][a_i]["n_points"]))
                            away_yes = True
                            a_i += 1
                        else:  # Nothing was printed need more alignmemt.
                            print('{:<54}{} {}'.format("",
                                                       point_scorers["away"][a_i]["name"], point_scorers["away"][a_i]["n_points"]))
                            away_yes = True
                            a_i += 1

                    if not away_yes and home_yes:  # Only printed home and not away, need new line.
                        print()

    global_cnt += len(started)


def print_finished(finished, team_list, get_data_bootstrap, player_cache):
    global global_cnt
    print("Finished:")
    for g in range(len(finished)):
        print('\n\n{:<5} {:>17}'.format(
            str(g + 1) + ".", team_list[finished[g]["team_h"]]), end="")
        print('{:<12} {} - {}'.format("",
                                      finished[g]["team_h_score"], finished[g]["team_a_score"]), end="")
        print('{:<12} {}'.format("", team_list[finished[g]["team_a"]]))

        for k in [0, 2, 1, 4, 5, 6, 9]:

            # Goals, assites then own goals.
            h_i, a_i = 0, 0
            h_points = len(finished[g]["stats"][k]["h"])
            a_points = len(finished[g]["stats"][k]["a"])

            if h_points > 0 or a_points > 0:  # Only print if some points were scored.
                if k == 0:
                    print('{:<36}{}'.format("", "Goals"))
                elif k == 2:
                    print('{:<36}{}'.format("", "Own goals"))
                elif k == 1:
                    print('{:<36}{}'.format("", "Assists"))
                elif k == 4:
                    print('{:<36}{}'.format("", "Penalties missed"))
                elif k == 5:
                    print('{:<36}{}'.format("", "Yellow cards"))
                elif k == 6:
                    print('{:<36}{}'.format("", "Red cards"))
                elif k == 9:
                    print('{:<36}{}'.format("", "Bonus"))

                if k == 9:

                    if finished[g]["finished"] == False:

                        bonus_points = find_prelim_bonus(finished[g])
                        # find_prelim_bonus returns data in wrong format, need to find team and name of player. Before printing.
                        temp_all = [{"value": v, "element": k}
                                    for k, v in bonus_points.items()]
                        home_team_id = finished[g]["team_h"]
                        away_team_id = finished[g]["team_a"]

                        point_scorers = {"home": [], "away": []}

                        for k in range(len(temp_all)):
                            ID = temp_all[k]["element"]
                            if ID in player_cache:
                                team_id = player_cache[ID]["team_id"]
                                if team_id == home_team_id:
                                    point_scorers["home"].append({
                                        "id": ID,
                                        "n_points": temp_all[k]["value"],
                                        "name": player_cache[ID]["name"],
                                        "team_id": player_cache[ID]["team_id"]
                                    })
                                elif team_id == away_team_id:
                                    point_scorers["away"].append({
                                        "id": ID,
                                        "n_points": temp_all[k]["value"],
                                        "name": player_cache[ID]["name"],
                                        "team_id": player_cache[ID]["team_id"]
                                    })
                                else:
                                    print(
                                        "Error, player ID doesnt belong to any of the teams.")
                                    sys.exit(1)
                            else:
                                for j in range(len(get_data_bootstrap['elements'])):
                                    if get_data_bootstrap['elements'][j]['id'] == ID:
                                        player_cache[ID] = {
                                            "name": get_data_bootstrap['elements'][j]["web_name"],
                                            "team_id": get_data_bootstrap['elements'][j]["team"]
                                        }

                                        team_id = player_cache[ID]["team_id"]

                                        if team_id == home_team_id:
                                            point_scorers["home"].append({
                                                "id": ID,
                                                "n_points": temp_all[k]["value"],
                                                "name": player_cache[ID]["name"],
                                                "team_id": player_cache[ID]["team_id"]
                                            })

                                        elif team_id == away_team_id:
                                            point_scorers["away"].append({
                                                "id": ID,
                                                "n_points": temp_all[k]["value"],
                                                "name": player_cache[ID]["name"],
                                                "team_id": player_cache[ID]["team_id"]
                                            })
                                        else:
                                            print(
                                                "Error, player ID doesnt belong to any of the teams.")
                                            sys.exit(1)
                                        break

                        h_points = len(point_scorers["home"])
                        a_points = len(point_scorers["away"])
                    else:  # Can use official bonus, no need for computation.
                        point_scorers = findNames(finished[g]["stats"][k - 1]["h"],
                                                  finished[g]["stats"][k - 1]["a"], get_data_bootstrap, player_cache)
                        h_points = len(point_scorers["home"])
                        a_points = len(point_scorers["away"])

                else:
                    point_scorers = findNames(finished[g]["stats"][k]["h"],
                                              finished[g]["stats"][k]["a"], get_data_bootstrap, player_cache)

                while h_i < h_points or a_i < a_points:
                    away_yes = False
                    home_yes = False

                    if h_i < h_points:
                        home_yes = True
                        print('{:>21} {}'.format(
                            point_scorers["home"][h_i]["name"], point_scorers["home"][h_i]["n_points"]), end='')
                        h_i += 1

                    if a_i < a_points:
                        if home_yes:  # Just printed a home player need alignment for away.
                            print('{:<31}{} {}'.format("",
                                                       point_scorers["away"][a_i]["name"], point_scorers["away"][a_i]["n_points"]))
                            away_yes = True
                            a_i += 1
                        else:  # Nothing was printed need more alignmemt.
                            print('{:<54}{} {}'.format("",
                                                       point_scorers["away"][a_i]["name"], point_scorers["away"][a_i]["n_points"]))
                            away_yes = True
                            a_i += 1

                    if not away_yes and home_yes:  # Only printed home and not away, need new line.
                        print()

    global_cnt += len(finished)
